@@ -13,7 +13,7 @@ use Sys::Syslog qw(:macros);
 use LISM::Storage;
 use Data::Dumper;
 
-our $VERSION = '2.2.0';
+our $VERSION = '2.2.1';
 
 our $lism_master = 'lism_master';
 our $syncrdn = 'cn=sync';
@@ -23,6 +23,7 @@ our $syncInfoEntry = "objectClass: lismSync\n";
 our $syncInfoAttr = "lismSyncStatus";
 our $nosyncAttr = "lismSyncWrongEntry";
 our $syncDataAttr = "lismSyncWrongNode";
+our $syncFilterAttr = "lismSyncFilter";
 our $clusterrdn = 'cn=cluster';
 our $clusterEntry = "objectClass: lismCluster\n";
 our $masterAttr = "lismClusterMaster";
@@ -129,6 +130,8 @@ sub init
         return 1;
     }
 
+    $self->log(level => 'info', message => "LISM $VERSION starting");
+
     return $self->_startup();
 }
 
@@ -1322,7 +1325,7 @@ sub _setConfig
 sub _addCluster
 {
     my $self = shift;
-    my ($dname) = @_;
+    my ($dname, $nosync) = @_;
     my $cluster = $self->{cluster};
     my $rc;
 
@@ -1332,7 +1335,9 @@ sub _addCluster
         $cluster->{$dname}->{status} = 'busy';
         $self->_unlock(2);
 
-        $rc = $self->_setSyncInfo("$cluster_syncrdn,".$self->{_config}->{basedn}, ('DELETE', $syncDataAttr, $dname));
+        if (!$nosync) {
+            $rc = $self->_setSyncInfo("$cluster_syncrdn,".$self->{_config}->{basedn}, ('DELETE', $syncDataAttr, $dname));
+        }
 
         $self->_lock(2);
         if ($rc) {
@@ -1448,7 +1453,7 @@ sub _setClusterInfo
     my $modinfo = join(',', @list);
 
     # get synchronized data
-    my ($add_dnames) = ($modinfo =~ /ADD,$activeAttr,(.*),?(ADD|DELETE|REPLACE|)/i);
+    my ($add_dnames) = ($modinfo =~ /ADD,$activeAttr[^,]*,(.*),?(ADD|DELETE|REPLACE|)/i);
     my ($delete_dnames) = ($modinfo =~ /DELETE,$activeAttr,(.*),?(ADD|DELETE|REPLACE|)/i);
 
     if ($delete_dnames) {
@@ -1466,7 +1471,11 @@ sub _setClusterInfo
             }
 
             if (",$add_dnames," =~ /$dname,/i) {
-                $self->_addCluster($dname);
+                my $nosync = 0;
+                if ($modinfo =~ /,$activeAttr;opt-nosync,/) {
+                    $nosync = 1;
+                }
+                $self->_addCluster($dname, $nosync);
             }
         }
     }
@@ -1752,7 +1761,7 @@ sub _getSyncInfo
     }
 
     # get checked data
-    my (@check_dnames) = ($filterStr =~ /\($syncDataAttr=(.*)\)/gi);
+    my (@check_dnames) = ($filterStr =~ /\($syncDataAttr=([^)]*)\)/gi);
     if (@check_dnames) {
         foreach my $dname (keys %{$cluster}) {
             if (grep(/$dname/i, @check_dnames)) {
@@ -1763,12 +1772,21 @@ sub _getSyncInfo
         @check_data = keys %{$cluster};
     }
 
+    # get check filter
+    my ($checkfilter) = ($filterStr =~ /\($syncFilterAttr=(.*)\)/i);
+    if ($checkfilter) {
+        $checkfilter =~ s/\\28/(/;
+        $checkfilter =~ s/\\29/)/;
+    } else {
+        $checkfilter = "(objectClass=*)";
+    }
+
     my $syncentry;
     ($syncentry = $base) =~ s/^([^=]+)=([^,]+),.*/$1: $2/;
     $syncentry = "dn: $base\n$syncInfoEntry$syncentry\n";
 
     # get present entry list
-    $present_list = $self->_getPresentList();
+    $present_list = $self->_getPresentList($checkfilter);
     if ($base !~ /^$cluster_syncrdn/) {
         # check master data
         foreach my $dname (@check_data) {
@@ -1815,7 +1833,7 @@ sub _getSyncInfo
                     }
 
                     # get values from data storage
-                    my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0);
+                    my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, $checkfilter, 0);
                     if ($rc) {
                         $self->log(level => 'error', message => "Can't get values of $dname($rc)");
                         return ($rc, ());
@@ -1964,7 +1982,7 @@ sub _getSyncInfo
                     }
 
                     # get values from data storage
-                    my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0);
+                    my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, $checkfilter, 0);
                     if ($rc) {
                         $self->log(level => 'error', message => "Can't get values of $dname($rc)");
                         return ($rc, ());
@@ -2091,6 +2109,10 @@ sub _setSyncInfo
 
     # get synchronized data
     my ($sync_dnames) = ($modinfo =~ /DELETE,$syncDataAttr,(.*),?(ADD|DELETE|REPLACE|)/i);
+
+    # get check filter
+    my ($checkfilter) = ($modinfo =~ /REPLACE,$syncFilterAttr,(.*),?(ADD|DELETE|REPLACE|)/i);
+
     if ($sync_dnames) {
         foreach my $dname (keys %{$cluster}) {
             if (",$sync_dnames," =~ /$dname,/i) {
@@ -2098,15 +2120,19 @@ sub _setSyncInfo
             }
         }
     } else {
-        if ($modinfo !~ /REPLACE,$syncInfoAttr,sync/i) {
+        if (!$checkfilter && $modinfo !~ /REPLACE,$syncInfoAttr,sync/i) {
             return LDAP_UNWILLING_TO_PERFORM;
         }
         @sync_data = keys %{$cluster};
     }
 
+    if (!$checkfilter) {
+        $checkfilter = "(objectClass=*)";
+    }
+
     if ($dn !~ /^$cluster_syncrdn/) {
         # get present entry list
-        $present_list = $self->_getPresentList();
+        $present_list = $self->_getPresentList($checkfilter);
 
         foreach my $dname (@sync_data) {
             if ($dname eq $self->{master}->{current}) {
@@ -2152,7 +2178,7 @@ sub _setSyncInfo
                     }
 
                     # get values from data storage
-                    my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0);
+                    my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, $checkfilter, 0);
                     if ($rc) {
                         $self->log(level => 'error', message => "Can't get values of $dname($rc)");
                         return $rc;
@@ -2357,7 +2383,7 @@ sub _setSyncInfo
     if ($dn !~ /^$master_syncrdn/) {
         # get new present entry list
         undef $present_list;
-        $present_list = $self->_getPresentList();
+        $present_list = $self->_getPresentList($checkfilter);
 
         foreach my $dname (@sync_data) {
             if ($dname eq $self->{master}->{current}) {
@@ -2404,7 +2430,7 @@ sub _setSyncInfo
                     }
 
                     # get values from data storage
-                    my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0);
+                    my ($rc, @entries) = $self->_do_search($dbase, 2, 0, $sizeLimit, 0, $checkfilter, 0);
                     if ($rc) {
                         $self->log(level => 'error', message => "Can't get values of $dname($rc)");
                         return $rc;
@@ -2575,6 +2601,7 @@ sub _setSyncInfo
 sub _getPresentList
 {
     my $self = shift;
+    my ($filterStr) = @_;
     my $conf = $self->{_lism};
     my $cluster = $self->{cluster};
     my $present_list;
@@ -2607,7 +2634,7 @@ sub _getPresentList
                         next;
                     }
 
-                    my ($rc, @entries) = $self->_do_search($sbase, 2, 0, $sizeLimit, 0, 'objectClass=*', 0, ());
+                    my ($rc, @entries) = $self->_do_search($sbase, 2, 0, $sizeLimit, 0, $filterStr, 0, ());
                     for (my $i = 0; $i < @entries; $i++) {
                         my ($key) = ($entries[$i] =~ /^dn: (.*),$sbase\n/i);
                         if (!$key) {
