@@ -138,6 +138,12 @@ sub _objSearch
     DO: {
         my $lock;
         my $file;
+
+        if (!$self->_checkFile($oconf->{file}[0])) {
+            $rc = LDAP_OPERATIONS_ERROR;
+            last DO;
+        }
+
         if (!open($lock, "> $oconf->{file}[0].lock")) {
             $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
             $rc = LDAP_OPERATIONS_ERROR;
@@ -243,6 +249,11 @@ sub _objModify
     my $lock;
     my $file;
     my $tmp;
+
+    if (!$self->_checkFile($oconf->{file}[0])) {
+        return LDAP_OPERATIONS_ERROR;
+    }
+
     if (!open($lock, "> $oconf->{file}[0].lock")) {
         $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
         return LDAP_OPERATIONS_ERROR;
@@ -414,6 +425,11 @@ sub _objAdd
     my $lock;
     my $file;
     my $tmp;
+
+    if (!$self->_checkFile($oconf->{file}[0])) {
+        return LDAP_OPERATIONS_ERROR;
+    }
+
     if (!open($lock, "> $oconf->{file}[0].lock")) {
         $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
         return LDAP_OPERATIONS_ERROR;
@@ -546,6 +562,11 @@ sub _objDelete
     my $lock;
     my $file;
     my $tmp;
+
+    if (!$self->_checkFile($oconf->{file}[0])) {
+        return LDAP_OPERATIONS_ERROR;
+    }
+
     if (!open($lock, "> $oconf->{file}[0].lock")) {
         $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
         return LDAP_OPERATIONS_ERROR;
@@ -623,6 +644,11 @@ sub _getParentRdn
 
     my $lock;
     my $file;
+
+    if (!$self->_checkFile($oconf->{file}[0])) {
+        return LDAP_OPERATIONS_ERROR;
+    }
+
     if (!open($lock, "> $oconf->{file}[0].lock")) {
         $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].lock");
         return LDAP_OPERATIONS_ERROR;
@@ -646,6 +672,10 @@ sub _getParentRdn
     }
 
     my $poconf = $self->{object}{$obj->{parent}}->{conf};
+    if (!$self->_checkFile($poconf->{file}[0])) {
+        return LDAP_OPERATIONS_ERROR;
+    }
+
     if (!open($lock, "> $poconf->{file}[0].lock")) {
         $self->log(level => 'alert', message => "Can't open $poconf->{file}[0].lock");
         return undef;
@@ -701,6 +731,11 @@ sub _getAttrValues
 
     my $lock;
     my $file;
+
+    if (!$self->_checkFile($filename)) {
+        return LDAP_OPERATIONS_ERROR;
+    }
+
     if (!open($lock, "> $filename.lock")) {
         $self->log(level => 'alert', message => "Can't open $filename.lock");
         return undef;
@@ -790,6 +825,10 @@ sub _getAttrKeys
         my $lock;
         my $file;
 
+        if (!$self->_checkFile($oconf->{attr}{$attr}->{file}[0])) {
+            return LDAP_OPERATIONS_ERROR;
+        }
+
         if (!open($lock, "> $oconf->{attr}{$attr}->{file}[0].lock")) {
             $self->log(level => 'alert', message => "Can't open $oconf->{attr}{$attr}->{file}[0].lock");
             return (-1, ());
@@ -851,6 +890,24 @@ sub _getAttrKeys
     return ($rc, @attrkeys);
 }
 
+sub _checkFile
+{
+    my $self = shift;
+    my ($filename) = @_;
+
+    if (!-f $filename) {
+        my $file;
+
+        if (!open($file, "> $filename")) {
+            $self->log(level => 'alert', message => "Can't create $filename");
+            return 0;
+        }
+        close($file);
+    }
+
+    return 1;
+}
+
 =head1 SEE ALSO
 
 L<LISM>,
