@@ -171,11 +171,6 @@ sub start_server {
   unlink("$TEMPDIR/syncfail-SQL.log");
   unlink("$TEMPDIR/syncfail-CSV.log");
 
-  foreach my $file ('user.csv', 'group.csv', 'division.csv', 'department.csv', 'category.csv', 'company.csv') {
-    open CSV, "> $TESTCSV/$file";
-    close CSV;
-  }
-
   open SAVEOUT, ">&STDERR";
   open STDERR, "> /dev/null";
 
